@@ -1,28 +1,29 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-
+from __future__ import with_statement
+from decimal import Decimal
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.pyson import Not, Equal, Eval
-from decimal import Decimal
+from trytond.transaction import Transaction
+
 
 class TimesheetLine(ModelSQL, ModelView):
     _name = 'timesheet.line'
 
-    def compute_cost(self, cursor, user, line, context=None):
+    def compute_cost(self, line):
         employee_obj = self.pool.get('company.employee')
         currency_obj = self.pool.get('currency.currency')
 
-        cost_price = employee_obj.compute_cost_price(cursor, user,
-                line.employee.id, date=line.date, context=context)
+        cost_price = employee_obj.compute_cost_price(line.employee.id,
+                date=line.date)
 
         line_company = line.employee.company
         work_company = line.work.company
         if line_company.id != work_company.id and\
                 line_company.currency.id != work_company.currency.id:
 
-            cost_price = currency_obj.compute(cursor, user,
-                    line_company.currency, cost_price,
-                    work_company.currency, context=context)
+            cost_price = currency_obj.compute(line_company.currency,
+                    cost_price, work_company.currency)
 
         return Decimal(str(line.hours)) * cost_price
 
@@ -53,13 +54,13 @@ class Work(ModelSQL, ModelView):
     currency_digits = fields.Function(fields.Integer('Currency Digits',
         on_change_with=['company']), 'get_currency_digits')
 
-    def get_cost(self, cursor, user, ids, name, context=None):
+    def get_cost(self, ids, name):
         timesheet_line_obj = self.pool.get('timesheet.line')
-        all_ids = self.search(cursor, user, [
+        all_ids = self.search([
                 ('parent', 'child_of', ids),
-                ('active', '=', True)], context=context) + ids
+                ('active', '=', True)]) + ids
         all_ids = list(set(all_ids))
-        works = set(self.browse(cursor, user, all_ids, context=context))
+        works = set(self.browse(all_ids))
         res = {}
         id2work = {}
         leafs = set()
@@ -70,8 +71,7 @@ class Work(ModelSQL, ModelView):
 
             res[work.id] = Decimal('0')
             for ts_line in work.timesheet_lines:
-                res[work.id] += timesheet_line_obj.compute_cost(cursor, user,
-                        ts_line, context=context)
+                res[work.id] += timesheet_line_obj.compute_cost(ts_line)
 
         while leafs:
             for work_id in leafs:
@@ -90,12 +90,12 @@ class Work(ModelSQL, ModelView):
             leafs = next_leafs
         return res
 
-    def get_revenue(self, cursor, user, ids, name, context=None):
-        all_ids = self.search(cursor, user, [
+    def get_revenue(self, ids, name):
+        all_ids = self.search([
                 ('parent', 'child_of', ids),
-                ('active', '=', True)], context=context) + ids
+                ('active', '=', True)]) + ids
         all_ids = list(set(all_ids))
-        works = set(self.browse(cursor, user, all_ids, context=context))
+        works = set(self.browse(all_ids))
         res = {}
         id2work = {}
         leafs = set()
@@ -126,32 +126,28 @@ class Work(ModelSQL, ModelView):
             leafs = next_leafs
         return res
 
-    def get_currency_digits(self, cursor, user, ids, name, context=None):
+    def get_currency_digits(self, ids, name):
         res = {}
-        for work in self.browse(cursor, user, ids, context=context):
+        for work in self.browse(ids):
             res[work.id] = work.company.currency.digits
         return res
 
-    def on_change_with_currency_digits(self, cursor, user, vals, context=None):
+    def on_change_with_currency_digits(self, vals):
         company_obj = self.pool.get('company.company')
         if vals.get('company'):
-            company = company_obj.browse(cursor, user, vals['company'],
-                    context=context)
+            company = company_obj.browse(vals['company'])
             return company.currency.digits
         return 2
 
-    def default_currency_digits(self, cursor, user, context=None):
+    def default_currency_digits(self):
         company_obj = self.pool.get('company.company')
-        if context is None:
-            context = {}
-        company = None
-        if context.get('company'):
-            company = company_obj.browse(cursor, user, context['company'],
-                    context=context)
+        company = Transaction().context.get('company')
+        if company:
+            company = company_obj.browse(company)
             return company.currency.digits
         return 2
 
-    def on_change_product(self, cursor, user, vals, context=None):
+    def on_change_product(self, vals):
         product_obj = self.pool.get('product.product')
         user_obj = self.pool.get('res.user')
         company_obj = self.pool.get('company.company')
@@ -161,31 +157,27 @@ class Work(ModelSQL, ModelView):
         if not vals.get('product'):
             return {}
 
-        ctx = context and context.copy() or {}
+        context = {}
 
-        product = product_obj.browse(cursor, user, vals['product'],
-                context=context)
+        product = product_obj.browse(vals['product'])
 
         if vals.get('party'):
-            ctx2['customer'] = vals['party']
+            context['customer'] = vals['party']
 
-        uom_id = model_data_obj.get_id(cursor, user, 'product', 'uom_hour',
-                context=context)
-        hour_uom = uom_obj.browse(cursor, user, uom_id, context=context)
+        uom_id = model_data_obj.get_id('product', 'uom_hour')
+        hour_uom = uom_obj.browse(uom_id)
 
-        list_price = uom_obj.compute_price(cursor, user,
-                product.default_uom, product.list_price, hour_uom,
-                context=ctx)
+        with Transaction().set_context(context):
+            list_price = uom_obj.compute_price(product.default_uom,
+                    product.list_price, hour_uom)
 
         if vals.get('company'):
-            user_record = user_obj.browse(cursor, user, user, context=context)
-            if user_record.company.id != vals['company']:
-                company = company_obj.browse(cursor, user, vals['company'],
-                        context=context)
-                if user_record.company.currency.id != company.currency.id:
-                    list_price = currency_obj.compute(cursor, user,
-                            user_record.company.currency, list_price,
-                            company.currency, context=context)
+            user = user_obj.browse(Transaction().user)
+            if user.company.id != vals['company']:
+                company = company_obj.browse(vals['company'])
+                if user.company.currency.id != company.currency.id:
+                    list_price = currency_obj.compute(user.company.currency,
+                            list_price, company.currency)
 
         return {'list_price': list_price}
 Work()
